"""
Special errors collection
"""


class ContentError(ValueError):
    pass


class LogicError(ValueError):
    pass


class RandomError(ValueError):
    pass


class ShapeError(ValueError):
    pass
